
#ifndef __ACM_WELDING_SYMBOL_LIBRARY_H
#define __ACM_WELDING_SYMBOL_LIBRARY_H
//
//  (C) Copyright 2008 by Autodesk, Inc.  All rights reserved.
//
// Permission to use, copy, modify, and distribute this software in
// object code form for any purpose and without fee is hereby granted, 
// provided that the above copyright notice appears in all copies and 
// that both that copyright notice and the limited warranty and
// restricted rights notice below appear in all supporting 
// documentation.
//
// AUTODESK PROVIDES THIS PROGRAM "AS IS" AND WITH ALL FAULTS. 
// AUTODESK SPECIFICALLY DISCLAIMS ANY IMPLIED WARRANTY OF
// MERCHANTABILITY OR FITNESS FOR A PARTICULAR USE.  AUTODESK, INC. 
// DOES NOT WARRANT THAT THE OPERATION OF THE PROGRAM WILL BE
// UNINTERRUPTED OR ERROR FREE.
//
// Use, duplication, or disclosure by the U.S. Government is subject to 
// restrictions set forth in FAR 52.227-19 (Commercial Computer
// Software - Restricted Rights) and DFAR 252.227-7013(c)(1)(ii)
// (Rights in Technical Data and Computer Software), as applicable.
//

#include "SymbolLib.h"

//////////////////////////////////////////////////////////////////////////
// class AmdtWeldingLibObject
class DllImpExp AmdtWeldingLibObject : public AmdtSymbolLibObject
{
public:
  ACRX_DECLARE_MEMBERS(AmdtWeldingLibObject);
  AmdtWeldingLibObject();
  ~AmdtWeldingLibObject();

  Acm::ErrorStatus getOtherSideReqValue(Acm::Variable var, ACHAR *& retVal, bool isPrimary=true) const;
  Acm::ErrorStatus getArrowSideReqValue(Acm::Variable var, ACHAR *& retVal, bool isPrimary=true) const;

  // AcDbObject Interface
  virtual Acad::ErrorStatus dwgInFields (AcDbDwgFiler* pFiler);
  virtual Acad::ErrorStatus dwgOutFields(AcDbDwgFiler* pFiler) const;

  Acm::SubSymbolType type(bool isArrowSide, bool isPrimary) const;
  Acm::SubSymbolType areaDescType() const;
  Acm::WeldAssociation associationType() const;
  Adesk::Boolean fieldWeld() const;
  Acm::IdentLineType identificationLine() const;
  Adesk::Boolean closedTail() const;
  bool operator==(const AmdtWeldingLibObject& weldLibObj)const;

protected:
  virtual Acad::ErrorStatus subGetClassID(CLSID* pClsid) const;
};
MAKE_ACDBOPENOBJECT_FUNCTION(AmdtWeldingLibObject);
DBREF_DECLARE(AmdtWeldingLibObject)


////////////////////////////////////////////////////////////////
// class AmdtWeldingLibrary
class DllImpExp AmdtWeldingLibrary : public AmdtSymbolLibrary
{
public:
  ACRX_DECLARE_MEMBERS(AmdtWeldingLibrary);
  AmdtWeldingLibrary();

  virtual const ACHAR *name() const;

  // Retrieve stored item
  virtual Acad::ErrorStatus loadItem(const ACHAR *itemName, AcmSymbol *pSym);
  virtual Acad::ErrorStatus loadItem(const ACHAR *itemName, AcmSymbol *pSym, Adesk::UInt16 index);

  // Add new item
  virtual Acad::ErrorStatus appendItem(const ACHAR *itemName, AcmSymbol *pSym, AcDbObjectId &returnId, bool bCheckValidName=true);
  virtual Acad::ErrorStatus appendItem(const ACHAR *itemName, AcmSymbol *pSym, Adesk::UInt16 index, AcDbObjectId &returnId, bool bCheckValidName=true);
  virtual Acad::ErrorStatus appendItem(AmdtSymbolLibObject* pObj, AcDbObjectId& returnId, bool bCheckValidName=true);

  // Update existing item
  virtual Acad::ErrorStatus modifyItem(const ACHAR *itemName, AcmSymbol *pSym);
  virtual Acad::ErrorStatus modifyItem(const ACHAR *itemName, AcmSymbol *pSym, Adesk::UInt16 index);

protected:
  // AcDbObject Interface
  virtual Acad::ErrorStatus subGetClassID(CLSID* pClsid) const;
};
MAKE_ACDBOPENOBJECT_FUNCTION(AmdtWeldingLibrary);
DBREF_DECLARE(AmdtWeldingLibrary)

//////////////////////////////////////////////////////////////////////////
// Global function
DllImpExp bool getAcmWeldingLibrary(AcDbObjectId &libId, AcDbDatabase *pDb=NULL, bool bCreateIfNotExist=true);

#endif //